/* abs@well.com */

#include <stdio.h>
#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>
#include <avs/geom.h>
#include <avs/colormap.h>
/* IAC CODE CHANGE : #include <avs/avs_math.h> */
#include <avs/avs_math.h>

/* To avoid obfuscating the code, put in a fixed polyhedron size */
#define MAX_PLIST_SIZE 1000000

AVSinit_modules()
{
    int AC_latlon_to_mesh();
  
    AVSmodule_from_desc(AC_latlon_to_mesh);
}

int AC_latlon_to_mesh()
{
    int AC_latlon_to_mesh_compute();

    /* Set the module name and type */
    AVSset_module_name("latlon_to_mesh", MODULE_MAPPER);
  
    /* Create an output port for the result */
    AVScreate_output_port("Geometry", "geom");
  
    AVSadd_float_parameter("lat_deg_LL", 0.0, -90.0, 90.0);
    AVSadd_float_parameter("lat_min_LL", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lat_sec_LL", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lon_deg_LL", 0.0, -180.0, 180.0);
    AVSadd_float_parameter("lon_min_LL", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lon_sec_LL", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lat_deg_UR", 0.0, -90.0, 90.0);
    AVSadd_float_parameter("lat_min_UR", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lat_sec_UR", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lon_deg_UR", 0.0, -180.0, 180.0);
    AVSadd_float_parameter("lon_min_UR", 0.0, 0.0, 60.0);
    AVSadd_float_parameter("lon_sec_UR", 0.0, 0.0, 60.0);
  
    /* Tell avs what subroutine to call to do the compute */
    AVSset_compute_proc(AC_latlon_to_mesh_compute);
}

int AC_latlon_to_mesh_compute(GEOMedit_list *output,
    float *lat_deg_LL, float *lat_min_LL, float *lat_sec_LL,
    float *lon_deg_LL, float *lon_min_LL, float *lon_sec_LL,
    float *lat_deg_UR, float *lat_min_UR, float *lat_sec_UR,
    float *lon_deg_UR, float *lon_min_UR, float *lon_sec_UR)
{  
  GEOMobj *obj;
  float *verts, *colors;
  int index, flags, cindex;
  float color, r, g, b;

  int nnn, ppp, p1, p2, p3, p4;
  float vertex[MAX_PLIST_SIZE][3];

  float x0, y0;
  float x1, y1;
  float z;

#ifdef DEBUG
  fprintf(stderr, "*lat_deg_LL = %f\n", *lat_deg_LL);
  fprintf(stderr, "*lat_min_LL = %f\n", *lat_min_LL);
  fprintf(stderr, "*lat_sec_LL = %f\n", *lat_sec_LL);
  fprintf(stderr, "*lon_deg_LL = %f\n", *lon_deg_LL);
  fprintf(stderr, "*lon_min_LL = %f\n", *lon_min_LL);
  fprintf(stderr, "*lon_sec_LL = %f\n", *lon_sec_LL);
  fprintf(stderr, "*lat_deg_UR = %f\n", *lat_deg_UR);
  fprintf(stderr, "*lat_min_UR = %f\n", *lat_min_UR);
  fprintf(stderr, "*lat_sec_UR = %f\n", *lat_sec_UR);
  fprintf(stderr, "*lon_deg_UR = %f\n", *lon_deg_UR);
  fprintf(stderr, "*lon_min_UR = %f\n", *lon_min_UR);
  fprintf(stderr, "*lon_sec_UR = %f\n", *lon_sec_UR);
#endif DEBUG

  if (*lon_deg_LL >= 0.0) {
      x0 = *lon_deg_LL + *lon_min_LL/60.0 + *lon_sec_LL/3600.0;
      }
  else {
      x0 = *lon_deg_LL - *lon_min_LL/60.0 - *lon_sec_LL/3600.0;
      }
  if (*lon_deg_UR >= 0.0) {
      x1 = *lon_deg_UR + *lon_min_UR/60.0 + *lon_sec_UR/3600.0;
      }
  else {
      x1 = *lon_deg_UR - *lon_min_UR/60.0 - *lon_sec_UR/3600.0;
      }
  if (*lat_deg_LL >= 0.0) {
      y0 = *lat_deg_LL + *lat_min_LL/60.0 + *lat_sec_LL/3600.0;
      }
  else {
      y0 = *lat_deg_LL - *lat_min_LL/60.0 - *lat_sec_LL/3600.0;
      }
  if (*lat_deg_UR >= 0.0) {
      y1 = *lat_deg_UR + *lat_min_UR/60.0 + *lat_sec_UR/3600.0;
      }
  else {
      y1 = *lat_deg_UR - *lat_min_UR/60.0 - *lat_sec_UR/3600.0;
      }
  z = 0.0;

#ifdef DEBUG
  fprintf(stderr, "x0 = %f\n", x0);
  fprintf(stderr, "y0 = %f\n", y0);
  fprintf(stderr, "x1 = %f\n", x1);
  fprintf(stderr, "y1 = %f\n", y1);
  fprintf(stderr, "z = %f\n", z);
#endif DEBUG

   vertex[0][0] = x0;
   vertex[0][1] = y1;
   vertex[0][2] = z;
   vertex[1][0] = x0;
   vertex[1][1] = y0;
   vertex[1][2] = z;
   vertex[2][0] = x1;
   vertex[2][1] = y1;
   vertex[2][2] = z;
   vertex[3][0] = x1;
   vertex[3][1] = y0;
   vertex[3][2] = z;

   /* Create an empty polyhedron object */
   obj = GEOMcreate_mesh_with_data(NULL,vertex,NULL,NULL,2,2,
                                         GEOM_DONT_COPY_DATA);

   /* Create normals */
   GEOMgen_normals(obj,0);

   /* Create mesh uvs */
   GEOMcreate_mesh_uvs(obj,0.0,0.0,1.0,1.0);

/*******************/
  *output = GEOMinit_edit_list(*output);

#ifdef DEBUG
  fprintf(stderr, "checkpoint #1\n");
#endif DEBUG

  GEOMedit_geometry(*output,"latlon_mesh",obj);

#ifdef DEBUG
  fprintf(stderr, "checkpoint #2\n");
#endif DEBUG

  GEOMdestroy_obj(obj);
      
#ifdef DEBUG
  fprintf(stderr, "checkpoint #3\n");
#endif DEBUG

  return(1);
}

